@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- $Id: battstat_applet.c,v 1.123 2006/07/10 18:22:56 dobey Exp $
+ $Id: battstat_applet.c,v 1.126 2006/10/02 14:01:21 davyd Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -57,6 +57,7 @@
 
 #define GCONF_PATH ""
 
+static gboolean check_for_updates (gpointer data);
 static void about_cb( BonoboUIComponent *, ProgressData *, const char * );
 static void help_cb( BonoboUIComponent *, ProgressData *, const char * );
 
@@ -266,6 +267,34 @@ allocate_battery_colours( void )
   }
 }
 
+/* Our backends may be either event driven or poll-based.
+ * If they are event driven then we know this the first time we
+ * receive an event.
+ */
+static gboolean event_driven = FALSE;
+static GSList *instances;
+
+static void
+status_change_callback (void)
+{
+  GSList *instance;
+
+  for (instance = instances; instance; instance = instance->next)
+  {
+    ProgressData *battstat = instance->data;
+
+    if (battstat->timeout_id)
+    {
+      g_source_remove (battstat->timeout_id);
+      battstat->timeout_id = 0;
+    }
+
+    check_for_updates (battstat);
+  }
+
+  event_driven = TRUE;
+}
+
 /* The following two functions keep track of how many instances of the applet
    are currently running.  When the first instance is started, some global
    initialisation is done.  When the last instance exits, cleanup occurs.
@@ -275,28 +304,34 @@ allocate_battery_colours( void )
    as the process quits immediately when the last applet is removed (which
    it does.)
 */
-static int instances;
-
 static const char *
-static_global_initialisation( int no_hal )
+static_global_initialisation (int no_hal, ProgressData *battstat)
 {
+  gboolean first_time;
   const char *err;
 
-  if( instances++ )
+  first_time = !instances;
+
+  instances = g_slist_prepend (instances, battstat);
+
+  if (!first_time)
     return NULL;
 
   allocate_battery_colours();
   initialise_global_pixmaps();
   glade_init();
-  err = power_management_initialise( no_hal );
+  err = power_management_initialise (no_hal, status_change_callback);
 
   return err;
 }
 
 static void
-static_global_teardown(void)
+static_global_teardown (ProgressData *battstat)
 {
-  if( --instances )
+  instances = g_slist_remove (instances, battstat);
+
+  /* remaining instances... */
+  if (instances)
     return;
 
   /* instances == 0 */
@@ -534,13 +569,22 @@ battery_low_update_text( ProgressData *battstat, BatteryStatus *info )
     gtk_widget_set_size_request( GTK_WIDGET( battstat->battery_low_label ),
                                  size.width, size.height );
 
-  remaining = g_strdup_printf( ngettext(
+  if (info->minutes < 0 && !info->on_ac_power)
+  {
+	  /* we don't know the remaining time */
+	  remaining = g_strdup_printf (_("You have %d%% of your total battery "
+				  	 "capacity remaining."), info->percent);
+  }
+  else
+  {
+	  remaining = g_strdup_printf( ngettext(
                                  "You have %d minute of battery power "
 				   "remaining (%d%% of the total capacity).",
                                  "You have %d minutes of battery power "
 				   "remaining (%d%% of the total capacity).",
                                  info->minutes ),
                                info->minutes,info->percent );
+  }
 
   if( is_suspend_unavailable() )
     /* TRANSLATORS: this is a list, it is left as a single string
@@ -963,7 +1007,7 @@ possibly_update_status_icon( ProgressData *battstat, BatteryStatus *info )
 /* Gets called as a gtk_timeout once per second.  Checks for updates and
    makes any changes as appropriate.
  */
-static gint
+static gboolean
 check_for_updates( gpointer data )
 {
   ProgressData *battstat = data;
@@ -975,6 +1019,30 @@ check_for_updates( gpointer data )
   if( (err = power_management_getinfo( &info )) )
     battstat_error_dialog( battstat->applet, err );
 
+  if (!event_driven)
+  {
+    int timeout;
+
+    /* if on AC and not even driven scale back the polls to once every 10 */
+    if (info.on_ac_power)
+      timeout = 10000;
+    else
+      timeout = 1000;
+
+    if (timeout != battstat->timeout)
+    {
+      battstat->timeout = timeout;
+
+      if (battstat->timeout_id)
+        g_source_remove (battstat->timeout_id);
+
+      battstat->timeout_id = g_timeout_add (battstat->timeout,
+                                            check_for_updates,
+                                            battstat);
+    }
+  }
+
+
   possibly_update_status_icon( battstat, &info );
 
   if (!info.on_ac_power &&
@@ -1090,7 +1158,8 @@ destroy_applet( GtkWidget *widget, ProgressData *battstat )
   if( battstat->battery_low_dialog )
     battery_low_dialog_destroy( battstat );
 
-  gtk_timeout_remove( battstat->pixtimer );
+  if (battstat->timeout_id)
+    g_source_remove (battstat->timeout_id);
 
   if( battstat->pixgc )
     g_object_unref( G_OBJECT(battstat->pixgc) );
@@ -1102,7 +1171,7 @@ destroy_applet( GtkWidget *widget, ProgressData *battstat )
 
   g_free( battstat );
 
-  static_global_teardown();
+  static_global_teardown (battstat);
 }
 
 /* Common function invoked by the 'Help' context menu item and the 'Help'
@@ -1548,11 +1617,6 @@ battstat_applet_fill (PanelApplet *applet)
 
   if (DEBUG) g_print("main()\n");
 
-  no_hal = panel_applet_gconf_get_bool( applet, "no_hal", NULL );
-
-  if( (err = static_global_initialisation( no_hal )) )
-    battstat_error_dialog( GTK_WIDGET (applet), err );
-
   gtk_window_set_default_icon_name ("battery");
   
   panel_applet_add_preferences (applet, "/schemas/apps/battstat-applet/prefs",
@@ -1574,6 +1638,8 @@ battstat_applet_fill (PanelApplet *applet)
   battstat->battery_low_label = NULL;
   battstat->about_dialog = NULL;
   battstat->pixgc = NULL;
+  battstat->timeout = -1;
+  battstat->timeout_id = 0;
 
   /* The first received size_allocate event will cause a reconfigure. */
   battstat->height = -1;
@@ -1583,8 +1649,6 @@ battstat_applet_fill (PanelApplet *applet)
   create_layout (battstat);
   setup_text_orientation( battstat );
 
-  battstat->pixtimer = gtk_timeout_add (1000, check_for_updates, battstat);
-
   panel_applet_setup_menu_from_file (PANEL_APPLET (battstat->applet), 
   			             DATADIR,
                                      "GNOME_BattstatApplet.xml",
@@ -1609,6 +1673,11 @@ battstat_applet_fill (PanelApplet *applet)
 	  atk_object_set_description(atk_widget, _("Monitor a laptop's remaining power"));
   }
 
+  no_hal = panel_applet_gconf_get_bool( applet, "no_hal", NULL );
+
+  if ((err = static_global_initialisation (no_hal, battstat)))
+    battstat_error_dialog (GTK_WIDGET (applet), err);
+
   return TRUE;
 }
 
