@@ -1,4 +1,4 @@
-/* $Id: gweather-about.c,v 1.24 2006/01/30 01:06:07 philipl Exp $ */
+/* $Id: gweather-about.c,v 1.25 2006/08/21 16:44:35 davyd Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -54,6 +54,6 @@ void gweather_about_run (GWeatherApplet *gw_applet)
 	"authors",	authors,
 	"documenters",	documenters,
 	"translator-credits",	_("translator-credits"),
-	"logo-icon-name",	"stock_weather-storm",
+	"logo-icon-name",	"weather-storm",
 	NULL);
 }
