@@ -1,4 +1,4 @@
-/* $Id: keyboard-drawing.c,v 1.43 2005/08/23 23:45:39 svu Exp $ */
+/* $Id: keyboard-drawing.c,v 1.48 2006/08/27 10:32:18 rodrigo Exp $ */
 /*
  * keyboard-drawing.c: implementation of a gtk+ widget that is a drawing of
  * the keyboard of the default display
@@ -532,106 +532,6 @@ set_key_label_in_layout (
 }
 
 
-/* normally, the unshifted (level 0) value goes on the bottom, but for some
- * keys, it goes on the top */
-static gboolean
-is_key_flipped (
-  KeyboardDrawing * drawing,
-  guint keycode)
-{
-  gint level, group;
-
-  if (!drawing->xkb) return FALSE;
-
-  for (group = 0; group < XkbKeyGroupsWidth (drawing->xkb, keycode); group++)
-    for (level = 0; level < XkbKeyGroupWidth (drawing->xkb, keycode, group);
-         level++)
-      switch (XkbKeySymEntry (drawing->xkb, keycode, level, group))
-        {
-        case GDK_Escape:
-        case GDK_F1:
-        case GDK_F2:
-        case GDK_F3:
-        case GDK_F4:
-        case GDK_F5:
-        case GDK_F6:
-        case GDK_F7:
-        case GDK_F8:
-        case GDK_F9:
-        case GDK_F10:
-        case GDK_F11:
-        case GDK_F12:
-        case GDK_F13:
-        case GDK_F14:
-        case GDK_F15:
-        case GDK_F16:
-        case GDK_F17:
-        case GDK_F18:
-        case GDK_F19:
-        case GDK_F20:
-        case GDK_F21:
-        case GDK_F22:
-        case GDK_F23:
-        case GDK_F24:
-        case GDK_F25:
-        case GDK_F26:
-        case GDK_F27:
-        case GDK_F28:
-        case GDK_F29:
-        case GDK_F30:
-        case GDK_F31:
-        case GDK_F32:
-        case GDK_F33:
-        case GDK_F34:
-        case GDK_F35:
-        case GDK_Print:
-        case GDK_Scroll_Lock:
-        case GDK_Pause:
-        case GDK_Tab:
-        case GDK_Control_L:
-        case GDK_Control_R:
-        case GDK_Shift_L:
-        case GDK_Shift_R:
-        case GDK_Super_L:
-        case GDK_Super_R:
-        case GDK_Alt_L:
-        case GDK_Alt_R:
-        case GDK_Meta_L:
-        case GDK_Meta_R:
-        case GDK_Mode_switch:
-        case GDK_Menu:
-        case GDK_Return:
-        case GDK_BackSpace:
-        case GDK_Insert:
-        case GDK_Home:
-        case GDK_Page_Up:
-        case GDK_Page_Down:
-        case GDK_Delete:
-        case GDK_End:
-        case GDK_Up:
-        case GDK_Down:
-        case GDK_Left:
-        case GDK_Right:
-        case GDK_Num_Lock:
-        case GDK_KP_Divide:
-        case GDK_KP_Multiply:
-        case GDK_KP_Subtract:
-        case GDK_KP_Add:
-        case GDK_KP_Enter:
-        case GDK_KP_Page_Up:
-        case GDK_KP_Page_Down:
-        case GDK_KP_Home:
-        case GDK_KP_End:
-        case GDK_KP_Left:
-        case GDK_KP_Right:
-        case GDK_KP_Up:
-        case GDK_KP_Down:
-          return TRUE;
-        }
-
-  return FALSE;
-}
-
 static void
 draw_layout (
   KeyboardDrawing * drawing,
@@ -707,7 +607,6 @@ draw_key_label_helper (
   gint y,
   gint width,
   gint height,
-  gboolean flipped,
   gint padding)
 {
   gint old_size;
@@ -715,6 +614,10 @@ draw_key_label_helper (
 
   if (keysym == 0)
     return;
+#ifdef KBDRAW_DEBUG
+  printf ("keysym: %04X(%c) at glp: %d\n",
+          (unsigned)keysym, (char)keysym, (int)glp);
+#endif
 
   switch( glp )
   {
@@ -722,8 +625,7 @@ draw_key_label_helper (
     case KEYBOARD_DRAWING_POS_BOTTOMLEFT:
     {
       ycell = glp == KEYBOARD_DRAWING_POS_BOTTOMLEFT;
-      ycell = flipped ? 1 - ycell : ycell;
-	
+
       rotate_coordinate (x, y, x + padding,
                          y + padding + (height -
                                         2 * padding) * ycell * 4 / 7, angle,
@@ -735,8 +637,7 @@ draw_key_label_helper (
     case KEYBOARD_DRAWING_POS_BOTTOMRIGHT:
     {
       ycell = glp == KEYBOARD_DRAWING_POS_BOTTOMRIGHT;
-      ycell = flipped ? 1 - ycell : ycell;
-	
+
       rotate_coordinate (x, y, x + padding + (width - 2 * padding) * 4 / 7,
                          y + padding + (height -
                                         2 * padding) * ycell * 4 / 7, angle,
@@ -773,7 +674,6 @@ draw_key_label (
   gint xkb_height)
 {
   gint x, y, width, height;
-  gboolean flipped;
   gint padding;
   gint g, l, glp;
 
@@ -786,14 +686,13 @@ draw_key_label (
   width = xkb_to_pixmap_coord (drawing, xkb_origin_x + xkb_width) - x;
   height = xkb_to_pixmap_coord (drawing, xkb_origin_y + xkb_height) - y;
 
-  flipped = is_key_flipped (drawing, keycode);
-
   for (glp = KEYBOARD_DRAWING_POS_TOPLEFT; glp<KEYBOARD_DRAWING_POS_TOTAL; glp++ )
     {
       if (drawing->groupLevels[glp] == NULL)
         continue;
       g = drawing->groupLevels[glp]->group;
       l = drawing->groupLevels[glp]->level;
+
       if (g < 0 || g >= XkbKeyNumGroups (drawing->xkb, keycode))
         continue;
       if (l < 0 || l >= XkbKeyGroupWidth (drawing->xkb, keycode, g))
@@ -809,7 +708,7 @@ draw_key_label (
                                    &mods_rtrn,
                                    &keysym))
             {
-              draw_key_label_helper (drawing, keysym, angle, glp, x, y, width, height, flipped, padding);  
+              draw_key_label_helper (drawing, keysym, angle, glp, x, y, width, height, padding);  
               /* reverse y order */
             }
         }
@@ -819,7 +718,7 @@ draw_key_label (
 
           keysym = XkbKeySymEntry (drawing->xkb, keycode, l, g);
 
-          draw_key_label_helper (drawing, keysym, angle, glp, x, y, width, height, flipped, padding);  
+          draw_key_label_helper (drawing, keysym, angle, glp, x, y, width, height, padding);  
           /* reverse y order */
         }
     }
@@ -1174,12 +1073,12 @@ key_event (
     {
       g_signal_emit (drawing, keyboard_drawing_signals[BAD_KEYCODE], 0,
                      event->hardware_keycode);
-      return FALSE;
+      return TRUE;
     }
 
   if ((event->type == GDK_KEY_PRESS && key->pressed) ||
       (event->type == GDK_KEY_RELEASE && !key->pressed))
-    return FALSE;
+    return TRUE;
   /* otherwise this event changes the state we believed we had before */
 
   key->pressed = (event->type == GDK_KEY_PRESS);
@@ -1188,7 +1087,7 @@ key_event (
 
   invalidate_key_region (drawing, key);
 
-  return FALSE;
+  return TRUE;
 }
 
 static gint
@@ -1422,7 +1321,7 @@ init_keys_and_doodads (
         }
     }
 
-  g_list_sort (drawing->keyboard_items,
+  drawing->keyboard_items = g_list_sort (drawing->keyboard_items,
                (GCompareFunc) compare_keyboard_item_priorities);
 }
 
@@ -1589,6 +1488,8 @@ destroy (
       g_source_remove (drawing->timeout);
       drawing->timeout = 0;
     }
+
+  g_object_unref (drawing->pixmap);
 }
 
 static void
