@@ -123,6 +123,7 @@ void
 stickynotes_applet_init (PanelApplet *panel_applet)
 {	
 	GnomeClient *client;
+	int timeout;
 
 	stickynotes = g_new(StickyNotes, 1);
 
@@ -132,10 +133,11 @@ stickynotes_applet_init (PanelApplet *panel_applet)
 	/* Register size for icons */
 	gtk_icon_size_register ("stickynotes_icon_size", 8,8);
 
-	gtk_window_set_default_icon_name ("stock_notes");
+	gtk_window_set_default_icon_name ("gnome-sticky-notes-applet");
 
 	stickynotes->icon_normal = gtk_icon_theme_load_icon (
-			gtk_icon_theme_get_default (), "stock_notes",
+			gtk_icon_theme_get_default (),
+			"gnome-sticky-notes-applet",
 			48, 0, NULL);
 
 	stickynotes->icon_prelight = gdk_pixbuf_new (
@@ -175,10 +177,16 @@ stickynotes_applet_init (PanelApplet *panel_applet)
 	stickynotes_load (gtk_widget_get_screen (GTK_WIDGET (panel_applet)));
 	
 	/* Auto-save every so minutes (default 5) */
-	g_timeout_add (1000 * 60 * gconf_client_get_int(stickynotes->gconf,
-				GCONF_PATH "/settings/autosave_time", NULL),
+	timeout = gconf_client_get_int (stickynotes->gconf,
+				GCONF_PATH "/settings/autosave_time", NULL);
+	if (timeout <= 0)
+	{
+		g_warning ("No default timeout, is the schema installed?");
+		timeout = 5;
+	}
+	g_timeout_add (1000 * 60 * timeout,
 		      (GSourceFunc) applet_save_cb, NULL);
-	g_timeout_add (100, applet_check_click_on_desktop_cb, NULL);
+	install_check_click_on_desktop ();
 }
 
 /* Initialize Sticky Notes Icons */
@@ -550,21 +558,24 @@ stickynotes_applet_update_prefs (void)
 
 void stickynotes_applet_update_menus(void)
 {
+	GList *l;
+	gboolean inconsistent = FALSE;
+
 	gboolean locked = gconf_client_get_bool(stickynotes->gconf, GCONF_PATH "/settings/locked", NULL);	
 	gboolean locked_writable = gconf_client_key_is_writable(stickynotes->gconf, GCONF_PATH "/settings/locked", NULL);
-	
-	gboolean inconsistent = FALSE;
-	gint i;
 
-	for (i = 0; i < g_list_length(stickynotes->notes) && !inconsistent; i++) {
-		StickyNote *note = g_list_nth_data(stickynotes->notes, i);
+	for (l = stickynotes->notes; l != NULL; l = l->next) {
+		StickyNote *note = l->data;
 
-		if (note->locked != locked)
+		if (note->locked != locked) {
 			inconsistent = TRUE;
+			break;
+		}
 	}
 
-	for (i = 0; i < g_list_length(stickynotes->applets); i++) {
-		StickyNotesApplet *applet = g_list_nth_data(stickynotes->applets, i);
+	for (l = stickynotes->applets; l != NULL; l = l->next) {
+		StickyNotesApplet *applet = l->data;
+
 		BonoboUIComponent *popup = panel_applet_get_popup_component(PANEL_APPLET(applet->w_applet));
 		
 		bonobo_ui_component_set_prop(popup, "/commands/lock", "state", locked ? "1" : "0", NULL);
